@@ -15,7 +15,11 @@
 #define HAVE_PQSETCLIENTENCODING 1
 #define HAVE_PQLIBVERSION 1
 #define HAVE_PQPING 1
+#define HAVE_PQSETSINGLEROWMODE 1
 #define HAVE_RB_ENCDB_ALIAS 1
+#define HAVE_RB_THREAD_CALL_WITHOUT_GVL 1
+#define HAVE_RB_THREAD_CALL_WITH_GVL 1
+#define HAVE_RB_THREAD_FD_SELECT 1
 #define HAVE_CONST_PGRES_COPY_BOTH 1
 #define HAVE_CONST_PGRES_SINGLE_TUPLE 1
 #define HAVE_STRUCT_PGNOTIFY_EXTRA 1
