@@ -1,6 +1,6 @@
 /*
  * pg_result.c - PG::Result class extension
- * $Id: pg_result.c,v 237cc4f69f63 2012/06/17 17:20:28 ged $
+ * $Id: pg_result.c,v 76ebae01c937 2013/03/26 17:50:02 ged $
  *
  */
 
@@ -37,7 +37,7 @@ pg_new_result(PGresult *result, VALUE rb_pgconn)
 
 /*
  * call-seq:
- *    res.result -> nil
+ *    res.check -> nil
  *
  * Raises appropriate exception if PG::Result is in a bad state.
  */
@@ -67,6 +67,9 @@ pg_result_check( VALUE self )
 		case PGRES_COPY_IN:
 #ifdef HAVE_CONST_PGRES_COPY_BOTH
 		case PGRES_COPY_BOTH:
+#endif
+#ifdef HAVE_CONST_PGRES_SINGLE_TUPLE
+		case PGRES_SINGLE_TUPLE:
 #endif
 		case PGRES_EMPTY_QUERY:
 		case PGRES_COMMAND_OK:
@@ -144,10 +147,10 @@ pgresult_get(VALUE self)
 
 
 /********************************************************************
- * 
+ *
  * Document-class: PG::Result
  *
- * The class to represent the query result tuples (rows). 
+ * The class to represent the query result tuples (rows).
  * An instance of this class is created as the result of every query.
  * You may need to invoke the #clear method of the instance when finished with
  * the result for better memory performance.
@@ -159,7 +162,7 @@ pgresult_get(VALUE self)
  *    res.getvalue(0,0) # '1'
  *    res[0]['b']       # '2'
  *    res[0]['c']       # nil
- *  
+ *
  */
 
 /**************************************************************************
@@ -179,6 +182,7 @@ pgresult_get(VALUE self)
  * * +PGRES_BAD_RESPONSE+
  * * +PGRES_NONFATAL_ERROR+
  * * +PGRES_FATAL_ERROR+
+ * * +PGRES_COPY_BOTH+
  */
 static VALUE
 pgresult_result_status(VALUE self)
@@ -205,7 +209,7 @@ pgresult_res_status(VALUE self, VALUE status)
  * call-seq:
  *    res.error_message() -> String
  *
- * Returns the error message of the command as a string. 
+ * Returns the error message of the command as a string.
  */
 static VALUE
 pgresult_error_message(VALUE self)
@@ -236,7 +240,7 @@ pgresult_error_message(VALUE self)
  * * +PG_DIAG_SOURCE_FUNCTION+
  *
  * An example:
- * 
+ *
  *   begin
  *       conn.exec( "SELECT * FROM nonexistant_table" )
  *   rescue PG::Error => err
@@ -255,10 +259,10 @@ pgresult_error_message(VALUE self)
  *           result.error_field( PG::Result::PG_DIAG_SOURCE_FUNCTION ),
  *       ]
  *   end
- * 
+ *
  * Outputs:
- * 
- *   ["ERROR", "42P01", "relation \"nonexistant_table\" does not exist", nil, nil, 
+ *
+ *   ["ERROR", "42P01", "relation \"nonexistant_table\" does not exist", nil, nil,
  *    "15", nil, nil, nil, "path/to/parse_relation.c", "857", "parserOpenTable"]
  */
 static VALUE
@@ -327,10 +331,24 @@ pgresult_fname(VALUE self, VALUE index)
  * call-seq:
  *    res.fnumber( name ) -> Fixnum
  *
- * Returns the index of the field specified by the string _name_.
+ * Returns the index of the field specified by the string +name+.
+ * The given +name+ is treated like an identifier in an SQL command, that is,
+ * it is downcased unless double-quoted. For example, given a query result
+ * generated from the SQL command:
  *
- * Raises an ArgumentError if the specified _name_ isn't one of the field names;
- * raises a TypeError if _name_ is not a String.
+ *   result = conn.exec( %{SELECT 1 AS FOO, 2 AS "BAR"} )
+ *
+ * we would have the results:
+ *
+ *   result.fname( 0 )            # => "foo"
+ *   result.fname( 1 )            # => "BAR"
+ *   result.fnumber( "FOO" )      # => 0
+ *   result.fnumber( "foo" )      # => 0
+ *   result.fnumber( "BAR" )      # => ArgumentError
+ *   result.fnumber( %{"BAR"} )   # => 1
+ *
+ * Raises an ArgumentError if the specified +name+ isn't one of the field names;
+ * raises a TypeError if +name+ is not a String.
  */
 static VALUE
 pgresult_fnumber(VALUE self, VALUE name)
@@ -363,7 +381,7 @@ pgresult_ftable(VALUE self, VALUE column_number)
 	int col_number = NUM2INT(column_number);
 	PGresult *pgresult = pgresult_get(self);
 
-	if( col_number < 0 || col_number >= PQnfields(pgresult)) 
+	if( col_number < 0 || col_number >= PQnfields(pgresult))
 		rb_raise(rb_eArgError,"Invalid column index: %d", col_number);
 
 	n = PQftable(pgresult, col_number);
@@ -374,7 +392,7 @@ pgresult_ftable(VALUE self, VALUE column_number)
  * call-seq:
  *    res.ftablecol( column_number ) -> Fixnum
  *
- * Returns the column number (within its table) of the table from 
+ * Returns the column number (within its table) of the table from
  * which the column _column_number_ is made up.
  *
  * Raises ArgumentError if _column_number_ is out of range or if
@@ -388,7 +406,7 @@ pgresult_ftablecol(VALUE self, VALUE column_number)
 
 	int n;
 
-	if( col_number < 0 || col_number >= PQnfields(pgresult)) 
+	if( col_number < 0 || col_number >= PQnfields(pgresult))
 		rb_raise(rb_eArgError,"Invalid column index: %d", col_number);
 
 	n = PQftablecol(pgresult, col_number);
@@ -401,7 +419,7 @@ pgresult_ftablecol(VALUE self, VALUE column_number)
  *
  * Returns the format (0 for text, 1 for binary) of column
  * _column_number_.
- * 
+ *
  * Raises ArgumentError if _column_number_ is out of range.
  */
 static VALUE
@@ -410,7 +428,7 @@ pgresult_fformat(VALUE self, VALUE column_number)
 	PGresult *result = pgresult_get(self);
 	int fnumber = NUM2INT(column_number);
 	if (fnumber < 0 || fnumber >= PQnfields(result)) {
-		rb_raise(rb_eArgError, "Column number is out of range: %d", 
+		rb_raise(rb_eArgError, "Column number is out of range: %d",
 			fnumber);
 	}
 	return INT2FIX(PQfformat(result, fnumber));
@@ -424,14 +442,14 @@ pgresult_fformat(VALUE self, VALUE column_number)
  *
  * The integer returned is the internal +OID+ number (in PostgreSQL)
  * of the type. To get a human-readable value for the type, use the
- * returned OID and the field's #fmod value with the format_type() SQL 
+ * returned OID and the field's #fmod value with the format_type() SQL
  * function:
- * 
+ *
  *   # Get the type of the second column of the result 'res'
  *   typename = conn.
  *     exec( "SELECT format_type($1,$2)", [res.ftype(1), res.fmod(1)] ).
  *     getvalue( 0, 0 )
- * 
+ *
  * Raises an ArgumentError if _column_number_ is out of range.
  */
 static VALUE
@@ -449,9 +467,9 @@ pgresult_ftype(VALUE self, VALUE index)
  * call-seq:
  *    res.fmod( column_number )
  *
- * Returns the type modifier associated with column _column_number_. See 
+ * Returns the type modifier associated with column _column_number_. See
  * the #ftype method for an example of how to use this.
- * 
+ *
  * Raises an ArgumentError if _column_number_ is out of range.
  */
 static VALUE
@@ -461,7 +479,7 @@ pgresult_fmod(VALUE self, VALUE column_number)
 	int fnumber = NUM2INT(column_number);
 	int modifier;
 	if (fnumber < 0 || fnumber >= PQnfields(result)) {
-		rb_raise(rb_eArgError, "Column number is out of range: %d", 
+		rb_raise(rb_eArgError, "Column number is out of range: %d",
 			fnumber);
 	}
 	modifier = PQfmod(result,fnumber);
@@ -670,7 +688,7 @@ pgresult_oid_value(VALUE self)
  * call-seq:
  *    res[ n ] -> Hash
  *
- * Returns tuple _n_ as a hash. 
+ * Returns tuple _n_ as a hash.
  */
 static VALUE
 pgresult_aref(VALUE self, VALUE index)
@@ -710,33 +728,31 @@ pgresult_aref(VALUE self, VALUE index)
 	return tuple;
 }
 
-
 /*
  * call-seq:
- *    res.values -> Array
+ *    res.each_row { |row| ... }
  *
- * Returns all tuples as an array of arrays.
+ * Yields each row of the result. The row is a list of column values.
  */
 static VALUE
-pgresult_values(VALUE self)
+pgresult_each_row(VALUE self)
 {
 	PGresult* result = (PGresult*) pgresult_get(self);
 	int row;
 	int field;
 	int num_rows = PQntuples(result);
 	int num_fields = PQnfields(result);
-	VALUE rows[ num_rows ];
 
 	for ( row = 0; row < num_rows; row++ ) {
-		VALUE new_row[ num_fields ];
+		VALUE new_row = rb_ary_new2(num_fields);
 
 		/* populate the row */
 		for ( field = 0; field < num_fields; field++ ) {
 			if ( PQgetisnull(result, row, field) ) {
-				new_row[ field ] = Qnil;
+				rb_ary_store( new_row, field, Qnil );
 			}
 			else {
-				VALUE val = rb_tainted_str_new( PQgetvalue(result, row, field), 
+				VALUE val = rb_tainted_str_new( PQgetvalue(result, row, field),
 				                                PQgetlength(result, row, field) );
 
 #ifdef M17N_SUPPORTED
@@ -747,14 +763,13 @@ pgresult_values(VALUE self)
 					rb_enc_associate( val, rb_ascii8bit_encoding() );
 				}
 #endif
-				new_row[ field ] = val;
+				rb_ary_store( new_row, field, val );
 			}
 		}
-
-		rows[ row ] = rb_ary_new4( num_fields, new_row );
+		rb_yield( new_row );
 	}
 
-	return rb_ary_new4( num_rows, rows );
+	return Qnil;
 }
 
 
@@ -769,7 +784,7 @@ make_column_result_array( VALUE self, int col )
 	int rows = PQntuples( result );
 	int i;
 	VALUE val = Qnil;
-	VALUE results[ rows ];
+	VALUE results = rb_ary_new2( rows );
 
 	if ( col >= PQnfields(result) )
 		rb_raise( rb_eIndexError, "no column %d in result", col );
@@ -780,17 +795,17 @@ make_column_result_array( VALUE self, int col )
 
 #ifdef M17N_SUPPORTED
 		/* associate client encoding for text format only */
-		if ( 0 == PQfformat(result, col) ) { 
+		if ( 0 == PQfformat(result, col) ) {
 			ASSOCIATE_INDEX( val, self );
 		} else {
 			rb_enc_associate( val, rb_ascii8bit_encoding() );
 		}
 #endif
 
-		results[ i ] = val;
+		rb_ary_store( results, i, val );
 	}
 
-	return rb_ary_new4( rows, results );
+	return results;
 }
 
 
@@ -798,7 +813,7 @@ make_column_result_array( VALUE self, int col )
  *  call-seq:
  *     res.column_values( n )   -> array
  *
- *  Returns an Array of the values from the nth column of each 
+ *  Returns an Array of the values from the nth column of each
  *  tuple in the result.
  *
  */
@@ -821,7 +836,7 @@ static VALUE
 pgresult_field_values( VALUE self, VALUE field )
 {
 	PGresult *result = pgresult_get( self );
-	const char *fieldname = RSTRING_PTR( field );
+	const char *fieldname = StringValuePtr( field );
 	int fnum = PQfnumber( result, fieldname );
 
 	if ( fnum < 0 )
@@ -860,16 +875,16 @@ pgresult_fields(VALUE self)
 {
 	PGresult *result = pgresult_get( self );
 	int n = PQnfields( result );
-	VALUE fields[ n ];
+	VALUE fields = rb_ary_new2( n );
 	int i;
 
 	for ( i = 0; i < n; i++ ) {
 		VALUE val = rb_tainted_str_new2(PQfname(result, i));
 		ASSOCIATE_INDEX(val, self);
-		fields[ i ] = val;
+		rb_ary_store( fields, i, val );
 	}
 
-	return rb_ary_new4( n, fields );
+	return fields;
 }
 
 
@@ -916,7 +931,7 @@ init_pg_result()
 	rb_define_method(rb_cPGresult, "[]", pgresult_aref, 1);
 	rb_define_method(rb_cPGresult, "each", pgresult_each, 0);
 	rb_define_method(rb_cPGresult, "fields", pgresult_fields, 0);
-	rb_define_method(rb_cPGresult, "values", pgresult_values, 0);
+	rb_define_method(rb_cPGresult, "each_row", pgresult_each_row, 0);
 	rb_define_method(rb_cPGresult, "column_values", pgresult_column_values, 1);
 	rb_define_method(rb_cPGresult, "field_values", pgresult_field_values, 1);
 }
